% Autor:
% Datum: 15.05.2017

:- use_module(comsemPredicates).


% introducing empty stack
betaConvert(X,Y):-
   betaConvert(X,Y,[]).

% no change for variables
betaConvert(X,Y,[]):-
   var(X), !,
   Y=X.

% expression is application => push argument to stack
betaConvert(Expression,Result,Stack):-
   nonvar(Expression),
   Expression = app(Functor,Argument),
   nonvar(Functor),
   betaConvert(Functor,Result,[Argument|Stack]), !.

% expression is abstraction => pop argument from stack
betaConvert(Expression,Result,[X|Stack]):-
   nonvar(Expression),
   Expression = lam(X,Formula),
   betaConvert(Formula,Result,Stack), !.

% other expression or abstraction with empty stack
% => break down complex expression
betaConvert(Expression,Result,[]):-
   nonvar(Expression),
   \+((Expression= app(X,_), nonvar(X))),
   compose(Expression,Functor,SubExpressions),
   betaConvertList(SubExpressions,ResultSubExpressions),
   compose(Result,Functor,ResultSubExpressions).

% beta convert lists
betaConvertList([],[]).

betaConvertList([Expression|Others], [Result|Results]) :-
     betaConvert(Expression,Result),
     betaConvertList(Others,Results).




/*========================================================================
   Syntax-semantics rules
========================================================================*/

s(app(NP,VP))--> np(NP), vp(VP).

np(app(Det,Noun))--> det(Det), noun(Noun).

np(PN)--> pn(PN).

vp(IV)--> iv(IV).

vp(app(TV,NP))--> tv(TV), np(NP).


/*========================================================================
   Proper Names
========================================================================*/

pn(lam(P,app(P,vincent)))--> [vincent].

pn(lam(P,app(P,mia)))--> [mia].


/*========================================================================
   Transitive Verbs
========================================================================*/

tv(lam(X,lam(Y,app(X,lam(Z,love(Y,Z))))))--> [loves].
tv(lam(X,lam(Y,app(X,lam(Z,like(Y,Z))))))--> [likes].


/*========================================================================
   Intransitive Verbs
========================================================================*/

iv(lam(Y,snort(Y)))--> [snorts].
iv(lam(Y,walk(Y)))--> [walks].


/*========================================================================
   Determiners
========================================================================*/

det(lam(P,lam(Q,all(X,imp(app(P,X),app(Q,X))))))--> [every].

det(lam(P,lam(Q,some(X,and(app(P,X),app(Q,X))))))--> [a].


/*========================================================================
   Nouns
========================================================================*/

noun(lam(X,woman(X)))--> [woman].
noun(lam(X,man(X)))--> [man].

noun(lam(X,footmassage(X)))--> [foot,massage].


